package Contas;

import java.util.Vector;

public class Repositorio {

	private Classe[] classes;
	private int quantidadeDeAmostras;
	private String identificacaoVariavel; 

	public Repositorio (Vector vectorAmostras, String identificacaoVariavel){

		this.identificacaoVariavel = identificacaoVariavel;
		this.quantidadeDeAmostras = vectorAmostras.size();
		
		int[] arrayAmostras = new int[vectorAmostras.size()];
		arrayAmostras = carregarArray(vectorAmostras);

		criarClasses(arrayAmostras);

	}
	
	public Classe[] getClasses(){
		return this.classes;
	}

	private int[] carregarArray(Vector<Integer> vectorAmostras){

		int indice = 0;
		int[] arrayAmostras = new int[vectorAmostras.size()];

		while (indice < vectorAmostras.size()){

			arrayAmostras[indice] = vectorAmostras.get(indice);

			indice++;
		}

		ordenarAmostras(arrayAmostras);

		return arrayAmostras;

	}
	private void criarClasses(int[] amostras){

		classes = new Classe[calcularNumeroDeClasses(amostras.length)];

		double amplitude = calcularAmplitude(amostras);
		double tempLimite = amostras[0];

		for (int i=0, j=0; i< classes.length; i++){		

			classes[i] = new Classe(tempLimite, tempLimite + amplitude);
			tempLimite = tempLimite + amplitude;
			int frequencia = 0;    

			while (j<amostras.length && amostras[j]<=classes[i].getLimiteSuperior()){
				if (amostras[j]<classes[i].getLimiteSuperior()){
					frequencia = frequencia + 1;
				} else {
					if (i == classes.length-1 && amostras[j]==classes[i].getLimiteSuperior())
						frequencia = frequencia + 1;
				}

				j = j+1;
			}
						
			classes[i].setFrequencia(frequencia);
			
			if (i == 0){
				classes[i].setFrequenciaAcumulada(frequencia);
			
			} else{
				classes[i].setFrequenciaAcumulada(frequencia + classes[i-1].getFrequenciaAcumulada());
			}
			
			classes[i].setFrequenciaRelativa( classes[i].getFrequencia()/this.quantidadeDeAmostras);
			
		}

	}

	private void ordenarAmostras(int[] amostras){

		for (int i=0; i<amostras.length ; i++){

			for (int j=i; j<amostras.length; j++ ){

				if(amostras[i] > amostras[j]){

					int temp;
					temp = amostras[j];
					amostras[j] = amostras[i];
					amostras[i] = temp;
				}

			}
		}
	}
	
	
	
	public double calcularMediana( int[] amostras ){
		
		double ordem = this.quantidadeDeAmostras/2;
		boolean achou = false;
		Classe classeMd = null;
		int i;
		
		for (i =0; i < classes.length && !achou; i++){
		
			if (ordem < classes[i].getFrequenciaAcumulada()){
				classeMd = classes[i-1];
				achou = true;
			}
		}
		
		double conta = classeMd.getLimiteInferior() + (( (ordem - classes[i-1].getFrequenciaAcumulada()) * classeMd.getAmplitude())/classeMd.getFrequencia() );
				
		return conta;
		
	}
	
	public double calcularMedia( int[] amostras ){
		
		double soma = 0;
		
		for (int i=0; i<this.calcularNumeroDeClasses(amostras.length); i++){
			
			soma = (soma + (classes[i].getValorMedio() * classes[i].getFrequencia()));
		}
		
		return soma/(quantidadeDeAmostras-1);

	}
	
	public double calcularVariancia( int[] amostras){
		
		double soma = 0;
		double media = this.calcularMedia(amostras);
		
		
		for (int i=0; i<this.calcularNumeroDeClasses(amostras.length); i++){
			
			soma = soma + ( Math.pow( (classes[i].getValorMedio() - media), 2) * classes[i].getFrequencia() );
		}
		
		return soma/(quantidadeDeAmostras-1);
	}
	
	public double calcularDesvioPadrao(int[] amostras){
		
		return Math.sqrt(calcularVariancia(amostras));
	}

	private int calcularRange(int[] amostras){

		return amostras[ (amostras.length - 1) ] - amostras[0] ;
	}

	private int calcularNumeroDeClasses(int tamanho){

		return ( (int)(1 + 3.22*Math.log10(tamanho)) ) + 1; 

	}

	private double calcularAmplitude(int[] amostras){

		return ( (double)calcularRange(amostras)/calcularNumeroDeClasses(amostras.length) );
	}

	public static void main(String[] args) {

		Vector teste = new Vector ();

		Classe t = new Classe (5,2);
		System.out.println(t.getFrequencia());
		teste.add(5);
		teste.add(8);
		teste.add(9);
		teste.add(11);
		teste.add(20);
		teste.add(8);
		teste.add(1);
		teste.add(6);
		teste.add(18);
		teste.add(2);
		teste.add(12);
		teste.add(10);
		teste.add(4);


		//Repositorio rep = new Repositorio (teste);
		//System.out.println(teste.size());
		//System.out.println(rep.classes[4].getFrequencia());

	}


}
